@@ -9,6 +9,13 @@
 #define __CURB_H
 
 #include <ruby.h>
+
+#ifdef HAVE_RUBY_IO_H
+#include "ruby/io.h"
+#else
+#include "rubyio.h" // ruby 1.8
+#endif
+
 #include <curl/curl.h>
 
 #include "curb_config.h"
@@ -20,11 +27,11 @@
 #include "curb_macros.h"
 
 // These should be managed from the Rake 'release' task.
-#define CURB_VERSION   "0.9.7"
-#define CURB_VER_NUM   907
+#define CURB_VERSION   "0.9.10"
+#define CURB_VER_NUM   9010
 #define CURB_VER_MAJ   0
 #define CURB_VER_MIN   9
-#define CURB_VER_MIC   7
+#define CURB_VER_MIC   10
 #define CURB_VER_PATCH 0
 
 
