@@ -11,7 +11,7 @@ class TestCurbCurlDownload < Test::Unit::TestCase
     dl_url = "http://127.0.0.1:9129/ext/curb_easy.c"
     dl_path = File.join(Dir::tmpdir, "dl_url_test.file")
 
-    curb = Curl::Easy.download(dl_url, dl_path)
+    Curl::Easy.download(dl_url, dl_path)
     assert File.exist?(dl_path)
     assert_equal File.read(File.join(File.dirname(__FILE__), '..','ext','curb_easy.c')), File.read(dl_path)
   ensure
@@ -23,7 +23,7 @@ class TestCurbCurlDownload < Test::Unit::TestCase
     dl_path = File.join(Dir::tmpdir, "dl_url_test.file")
     io = File.open(dl_path, 'wb')
 
-    curb = Curl::Easy.download(dl_url, io)
+    Curl::Easy.download(dl_url, io)
     assert io.closed?
     assert File.exist?(dl_path)
     assert_equal File.read(File.join(File.dirname(__FILE__), '..','ext','curb_easy.c')), File.read(dl_path)
@@ -49,7 +49,7 @@ class TestCurbCurlDownload < Test::Unit::TestCase
     # Download remote source
     begin
       reader.close
-      curb = Curl::Easy.download(dl_url, writer)
+      Curl::Easy.download(dl_url, writer)
       Process.wait
     ensure
       writer.close rescue IOError # if the stream has already been closed, which occurs in Easy::download
