@@ -10,6 +10,66 @@ class TestCurbCurlEasy < Test::Unit::TestCase
     Curl.reset
   end
 
+  def test_curlopt_stderr_with_file
+    # does not work with Tempfile directly
+    path = Tempfile.new('curb_test_curlopt_stderr').path
+    File.open(path, 'w') do |file|
+      easy = Curl::Easy.new(TestServlet.url)
+      easy.verbose = true
+      easy.setopt(Curl::CURLOPT_STDERR, file)
+      easy.perform
+    end
+    output = File.read(path)
+
+    assert_match('HTTP/1.1 200 OK ', output)
+    assert_match('Host: 127.0.0.1:9129', output)
+  end
+
+  def test_curlopt_stderr_with_io
+    path = Tempfile.new('curb_test_curlopt_stderr').path
+    fd = IO.sysopen(path, 'w')
+    io = IO.for_fd(fd)
+
+    easy = Curl::Easy.new(TestServlet.url)
+    easy.verbose = true
+    easy.setopt(Curl::CURLOPT_STDERR, io)
+    easy.perform
+
+
+    output = File.read(path)
+
+    assert_match(output, 'HTTP/1.1 200 OK')
+    assert_match(output, 'Host: 127.0.0.1:9129')
+  end
+
+  def test_curlopt_stderr_fails_with_tempdir
+    Tempfile.open('curb_test_curlopt_stderr') do |tempfile|
+      easy = Curl::Easy.new(TestServlet.url)
+
+      assert_raise(TypeError) do
+        easy.setopt(Curl::CURLOPT_STDERR, tempfile)
+      end
+    end
+  end
+
+  def test_curlopt_stderr_fails_with_stringio
+    stringio = StringIO.new
+    easy = Curl::Easy.new(TestServlet.url)
+
+    assert_raise(TypeError) do
+      easy.setopt(Curl::CURLOPT_STDERR, stringio)
+    end
+  end
+
+  def test_curlopt_stderr_fails_with_string
+    string = String.new
+    easy = Curl::Easy.new(TestServlet.url)
+
+    assert_raise(TypeError) do
+      easy.setopt(Curl::CURLOPT_STDERR, string)
+    end
+  end
+
   def test_exception
     begin
       Curl.get('NOT_FOUND_URL')
@@ -28,8 +88,6 @@ class TestCurbCurlEasy < Test::Unit::TestCase
           c = Curl.get($TEST_URL)
           assert_match(/^# DO NOT REMOVE THIS COMMENT/, c.body_str)
           assert_match(/^# DO NOT REMOVE THIS COMMENT/, c.body)
-          assert_equal "", c.header_str
-          assert_equal "", c.head
         end
       end
     end
@@ -41,7 +99,6 @@ class TestCurbCurlEasy < Test::Unit::TestCase
     assert_instance_of Curl::Easy, c = Curl::Easy.perform($TEST_URL)
     assert_match(/^# DO NOT REMOVE THIS COMMENT/, c.body_str)
     assert_match(/^# DO NOT REMOVE THIS COMMENT/, c.body)
-    assert_equal "", c.header_str
   end    
 
   def test_class_perform_02
@@ -49,7 +106,6 @@ class TestCurbCurlEasy < Test::Unit::TestCase
     assert_instance_of Curl::Easy, c = Curl::Easy.perform($TEST_URL) { |curl| curl.on_body { |d| data << d; d.length } }    
 
     assert_nil c.body_str
-    assert_equal "", c.header_str
     assert_match(/^# DO NOT REMOVE THIS COMMENT/, data)
   end    
 
@@ -144,7 +200,6 @@ class TestCurbCurlEasy < Test::Unit::TestCase
     c = Curl::Easy.new($TEST_URL)    
     assert_equal true, c.http_get
     assert_match(/^# DO NOT REMOVE THIS COMMENT/, c.body_str)
-    assert_equal "", c.header_str
   end    
 
   def test_get_02
@@ -156,7 +211,6 @@ class TestCurbCurlEasy < Test::Unit::TestCase
     assert_equal true, c.http_get    
     
     assert_nil c.body_str
-    assert_equal "", c.header_str
     assert_match(/^# DO NOT REMOVE THIS COMMENT/, data)
   end    
 
@@ -339,29 +393,69 @@ class TestCurbCurlEasy < Test::Unit::TestCase
     c.max_redirects = nil
     assert_nil c.max_redirects
   end
-  
+
+  def test_timeout_with_floats
+    c = Curl::Easy.new($TEST_URL)
+
+    c.timeout = 1.5
+    assert_equal 1500, c.timeout_ms
+    assert_equal 1.5, c.timeout
+  end
+
+  def test_timeout_with_negative
+    c = Curl::Easy.new($TEST_URL)
+
+    c.timeout = -1.5
+    assert_equal 0, c.timeout
+    assert_equal 0, c.timeout_ms
+
+    c.timeout = -4.8
+    assert_equal 0, c.timeout
+    assert_equal 0, c.timeout_ms
+  end
+
   def test_timeout_01
     c = Curl::Easy.new($TEST_URL)
-    
-    assert_nil c.timeout
-    
+
+    assert_equal 0, c.timeout
+
     c.timeout = 3
     assert_equal 3, c.timeout
-    
-    c.timeout = nil
-    assert_nil c.timeout
+
+    c.timeout = 0
+    assert_equal 0, c.timeout
   end
 
   def test_timeout_ms_01
     c = Curl::Easy.new($TEST_URL)
 
-    assert_nil c.timeout_ms
+    assert_equal 0, c.timeout_ms
 
     c.timeout_ms = 100
     assert_equal 100, c.timeout_ms
 
     c.timeout_ms = nil
-    assert_nil c.timeout_ms
+    assert_equal 0, c.timeout_ms
+  end
+
+  def test_timeout_ms_with_floats
+    c = Curl::Easy.new($TEST_URL)
+
+    c.timeout_ms = 55.5
+    assert_equal 55, c.timeout_ms
+    assert_equal 0.055, c.timeout
+  end
+
+  def test_timeout_ms_with_negative
+    c = Curl::Easy.new($TEST_URL)
+
+    c.timeout_ms = -1.5
+    assert_equal 0, c.timeout
+    assert_equal 0, c.timeout_ms
+
+    c.timeout_ms = -4.8
+    assert_equal 0, c.timeout
+    assert_equal 0, c.timeout_ms
   end
 
   def test_connect_timeout_01
@@ -608,7 +702,7 @@ class TestCurbCurlEasy < Test::Unit::TestCase
   def test_cookielist
     c = Curl::Easy.new TestServlet.url
     c.enable_cookies = true
-    c.post_body = URI.encode_www_form c: 'somename=somevalue'
+    c.post_body = URI.encode_www_form('c' => 'somename=somevalue')
     assert_nil c.cookielist
     c.perform
     assert_match(/somevalue/, c.cookielist.join(''))
